'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('vm2');

const NodeVM = _require.NodeVM;

var _require2 = require('bluebird');

const Promise = _require2.Promise;

const Instagram = require('instagram-private-api').V1;
const Command = require('./command.js');
const RandomString = require('randomstring');
const _ = require('lodash');
const util = require('util');

class CommandBox {
  constructor(commands) {
    this.pendingCommands = commands;
    this.finishedCommands = [];

    this.installedFunctions = {};

    this.vm = new NodeVM({
      console: 'inherit',
      sandbox: {
        Instagram,
        Promise,
        util,
        props: {}
      },
      require: {
        external: true
      }
    });

    // this.insertIntoSandbox({
    //     Instagram,
    //     Promise,
    //     props: {},
    // });
  }

  // insertIntoSandbox(sandbox) {
  //   const mergedSandBox = {
  //     ...this.vm.options.sandbox,
  //     ...sandbox,
  //   };

  //   this.vm.options.sandbox = mergedSandBox;
  // }

  currentSandbox() {
    return this.vm.sandbox;
  }

  getCurrentCommand() {
    return _.head(this.pendingCommands);
  }

  moveToNextCommand() {
    const currentCommand = this.getCurrentCommand();
    this.pendingCommands = _.drop(this.pendingCommands);
    this.finishedCommands.push(currentCommand);
  }

  installFunction(functionName, functionString) {
    const functionInSandBox = this.vm.run(`module.exports = ${ functionString }`);
    this.installedFunctions[functionName] = functionInSandBox;
  }

  runInstalledFunction(functionName) {
    const functionInSandBox = this.installedFunctions[functionName];

    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    return functionInSandBox.apply(this, args);
  }

  runCommand(command) {
    const script = command.script;
    const sandbox = command.sandbox;
    const context = this.vm._context;

    return new Promise((resolve, reject) => {
      const mergedSandBox = (0, _extends3.default)({}, this.vm.options.sandbox, sandbox, {
        resolve,
        reject
      });
      this.vm = new NodeVM({
        sandbox: mergedSandBox,
        context,
        console: 'inherit',
        require: {
          external: true
        }
      });

      this.vm.run(script, `${ RandomString.generate() }.js`);
    }).then(result => {
      command.success = true;
      command.result = result;
      return result;
    }, error => {
      command.success = false;
      command.error = error;
      throw error;
    });
  }

  runCurrentCommand() {
    return new Promise((resolve, reject) => {
      const command = this.getCurrentCommand();

      return this.runCommand(command).then(result => {
        this.moveToNextCommand();
        resolve(result);
      }, error => {
        this.moveToNextCommand();
        reject(error);
      });
    });
  }

  run() {
    const promisifyCommands = Promise.promisifyAll(this.pendingCommands);

    return Promise.each(promisifyCommands, command => {
      return this.runCommand(command);
    });
  }
}
exports.default = CommandBox;